@@ -14,11 +14,11 @@
 
 """Datasource action implemenations"""
 
-import logging
-
 from cliff import command
 from cliff import lister
 from cliff import show
+from oslo_log import log as logging
+from oslo_serialization import jsonutils
 import six
 
 from congressclient.common import parseractions
@@ -37,8 +37,9 @@ class ListDatasources(lister.Lister):
     def take_action(self, parsed_args):
         client = self.app.client_manager.congressclient
         data = client.list_datasources()['results']
-        columns = ['id', 'name', 'enabled', 'type', 'config']
-        formatters = {'Datasources': utils.format_list}
+        # Type is always None, so disabling it for now.
+        columns = ['id', 'name', 'enabled', 'driver', 'config']
+        formatters = {'config': utils.format_dict}
         return (columns,
                 (utils.get_dict_properties(s, columns,
                                            formatters=formatters)
@@ -54,17 +55,15 @@ class ListDatasourceTables(lister.Lister):
         parser = super(ListDatasourceTables, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar="<datasource-name>",
-            help="Name of the datasource")
+            metavar="<datasource>",
+            help="Name or ID of the datasource")
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
         client = self.app.client_manager.congressclient
-        results = client.list_datasources()
-        datasource_id = utils.get_resource_id_from_name(
-            parsed_args.datasource_name, results)
-        data = client.list_datasource_tables(datasource_id)['results']
+        name_or_id = parsed_args.datasource_name
+        data = client.list_datasource_tables(name_or_id)['results']
         columns = ['id']
         formatters = {'DatasourceTables': utils.format_list}
         return (columns,
@@ -82,22 +81,52 @@ class ShowDatasourceStatus(show.ShowOne):
         parser = super(ShowDatasourceStatus, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar="<datasource-name>",
-            help="Name of the datasource")
+            metavar="<datasource>",
+            help="Name or ID of the datasource")
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
         client = self.app.client_manager.congressclient
-
-        results = client.list_datasources()
-        datasource_id = utils.get_resource_id_from_name(
-            parsed_args.datasource_name, results)
-
+        datasource_id = parsed_args.datasource_name
         data = client.list_datasource_status(datasource_id)
         return zip(*sorted(six.iteritems(data)))
 
 
+class ShowDatasourceActions(lister.Lister):
+    """List supported actions for datasource."""
+
+    log = logging.getLogger(__name__ + '.ShowDatasourceActions')
+
+    def get_parser(self, prog_name):
+        parser = super(ShowDatasourceActions, self).get_parser(prog_name)
+        parser.add_argument(
+            'datasource_name',
+            metavar="<datasource>",
+            help="Name or ID of the datasource")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)' % parsed_args)
+        # as we know output it's long, limit column length here
+        if parsed_args.max_width == 0:
+            parsed_args.max_width = 40
+
+        client = self.app.client_manager.congressclient
+        datasource_id = parsed_args.datasource_name
+        data = client.list_datasource_actions(datasource_id)
+        formatters = {'args': utils.format_long_dict_list}
+        newdata = [{'action': x['name'],
+                    'args': x['args'],
+                    'description': x['description']}
+                   for x in data['results']]
+        columns = ['action', 'args', 'description']
+        return (columns, (utils.get_dict_properties(s,
+                                                    columns,
+                                                    formatters=formatters)
+                for s in newdata))
+
+
 class ShowDatasourceSchema(lister.Lister):
     """Show schema for datasource."""
 
@@ -107,16 +136,14 @@ class ShowDatasourceSchema(lister.Lister):
         parser = super(ShowDatasourceSchema, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar="<datasource-name>",
-            help="Name of the datasource")
+            metavar="<datasource>",
+            help="Name or ID of the datasource")
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
         client = self.app.client_manager.congressclient
-        results = client.list_datasources()
-        datasource_id = utils.get_resource_id_from_name(
-            parsed_args.datasource_name, results)
+        datasource_id = parsed_args.datasource_name
         data = client.show_datasource_schema(datasource_id)
         formatters = {'columns': utils.format_long_dict_list}
         newdata = [{'table': x['table_id'],
@@ -138,8 +165,8 @@ class ShowDatasourceTableSchema(lister.Lister):
         parser = super(ShowDatasourceTableSchema, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar="<datasource-name>",
-            help="Name of the datasource")
+            metavar="<datasource>",
+            help="Name or ID of the datasource")
         parser.add_argument(
             'table_name',
             metavar="<table-name>",
@@ -149,8 +176,9 @@ class ShowDatasourceTableSchema(lister.Lister):
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
         client = self.app.client_manager.congressclient
+        datasource_id = parsed_args.datasource_name
         data = client.show_datasource_table_schema(
-            parsed_args.datasource_name,
+            datasource_id,
             parsed_args.table_name)
         columns = ['name', 'description']
         return (columns,
@@ -167,8 +195,8 @@ class ListDatasourceRows(lister.Lister):
         parser = super(ListDatasourceRows, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar="<datasource-name>",
-            help="Name of the datasource to show")
+            metavar="<datasource>",
+            help="Name or ID of the datasource to show")
         parser.add_argument(
             'table',
             metavar="<table>",
@@ -178,10 +206,7 @@ class ListDatasourceRows(lister.Lister):
     def take_action(self, parsed_args):
         self.log.debug('take_action(%s)' % parsed_args)
         client = self.app.client_manager.congressclient
-
-        results = client.list_datasources()
-        datasource_id = utils.get_resource_id_from_name(
-            parsed_args.datasource_name, results)
+        datasource_id = parsed_args.datasource_name
         results = client.list_datasource_rows(datasource_id,
                                               parsed_args.table)['results']
         if results:
@@ -202,8 +227,8 @@ class ShowDatasourceTable(show.ShowOne):
         parser = super(ShowDatasourceTable, self).get_parser(prog_name)
         parser.add_argument(
             'datasource_name',
-            metavar='<datasource-name>',
-            help="Name of datasource")
+            metavar='<datasource>',
+            help="Name or ID of datasource")
         parser.add_argument(
             'table_id',
             metavar='<table-id>',
@@ -268,8 +293,67 @@ class DeleteDatasource(command.Command):
         parser = super(DeleteDatasource, self).get_parser(prog_name)
         parser.add_argument(
             'datasource',
-            metavar="<datasource-name>",
-            help="Name of the datasource to delete")
+            metavar="<datasource>",
+            help="Name or ID of the datasource to delete")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)' % parsed_args)
+        client = self.app.client_manager.congressclient
+        try:
+            datasource_id = parsed_args.datasource
+            client.delete_datasource(datasource_id)
+        except Exception:
+            # for backwards compatibility with pre-Ocata congress server,
+            # try old method of explicit conversion from name to UUID
+            results = client.list_datasources()
+            datasource_id = utils.get_resource_id_from_name(
+                parsed_args.datasource, results)
+            client.delete_datasource(datasource_id)
+
+
+class UpdateDatasourceRow(command.Command):
+    """Update rows to a datasource table."""
+
+    log = logging.getLogger(__name__ + '.UpdateDatasourceRow')
+
+    def get_parser(self, prog_name):
+        parser = super(UpdateDatasourceRow, self).get_parser(prog_name)
+        parser.add_argument(
+            'datasource',
+            metavar="<datasource>",
+            help="Name or ID of the datasource to Update")
+        parser.add_argument(
+            'table',
+            metavar="<table>",
+            help="Name or ID of the table to Update")
+        parser.add_argument(
+            'rows',
+            type=jsonutils.loads,
+            metavar="<rows>",
+            help=("List of Rows should be formmated json style."
+                  " ex. [[row1], [row2]]"))
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)' % parsed_args)
+        client = self.app.client_manager.congressclient
+        body = parsed_args.rows
+        client.update_datasource_rows(
+            parsed_args.datasource, parsed_args.table, body)
+
+
+class DatasourceRequestRefresh(command.Command):
+    """Trigger a datasource to poll."""
+
+    log = logging.getLogger(__name__ + '.DatasourceRequestRefresh')
+
+    def get_parser(self, prog_name):
+        parser = super(DatasourceRequestRefresh, self).get_parser(prog_name)
+        parser.add_argument(
+            'datasource',
+            metavar="<datasource>",
+            help="Name or ID of the datasource to poll")
         return parser
 
     def take_action(self, parsed_args):
@@ -278,4 +362,4 @@ class DeleteDatasource(command.Command):
         results = client.list_datasources()
         datasource_id = utils.get_resource_id_from_name(
             parsed_args.datasource, results)
-        client.delete_datasource(datasource_id)
+        client.request_refresh(datasource_id, {})
